@@ -1183,7 +1183,7 @@ template ArrayXd h3_ijk_vEc(const ArrayBase<ArrayXd>& A1,
                             int nthreads);
 
 template <typename DerivedA, typename DerivedB, typename DerivedC, typename DerivedD>
-inline void scale_in_htil3_pjk_mE(Eigen::Index j, Eigen::Index k,
+inline void scale_in_hxxx3_pjk_mE(Eigen::Index j, Eigen::Index k,
                                   const Eigen::Index m, const Eigen::Index n,
                                   const Eigen::Index p,
                                   typename DerivedA::Scalar &thr,
@@ -1202,13 +1202,14 @@ inline void scale_in_htil3_pjk_mE(Eigen::Index j, Eigen::Index k,
 // // [[Rcpp::export]]
 template <typename Derived>
 Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, Eigen::Dynamic>
-htil3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
+hxxx3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
              const Eigen::DiagonalMatrix<typename Derived::Scalar, Eigen::Dynamic>& A2,
              const Eigen::MatrixBase<Derived>& A3,
              const Eigen::Matrix<typename Derived::Scalar, Eigen::Dynamic, 1> mu,
              const Eigen::Index m, const Eigen::Index p,
              Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
-             const typename Derived::Scalar thr_margin, int nthreads) {
+             const typename Derived::Scalar thr_margin, int nthreads,
+             const typename Derived::Scalar c) {
 #ifdef _OPENMP
     if(nthreads <= 0) nthreads = std::max(omp_get_num_procs() / 2, 1);
     omp_set_num_threads(nthreads);
@@ -1232,10 +1233,11 @@ htil3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
             A1 * Gn.block(0, (i - 1) * n, n, n);
         MatrixXx::Map(gn.col(i).data(), n, 1).noalias() =
             Gn.block(0, i * n, n, n) * mu + A1 * gn.col(i - 1);
-        dks.ULSat(i, 0, 0, m + 1) = (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * i);
+        dks.ULSat(i, 0, 0, m + 1) =
+            (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * i);
         Gn.block(0, i * n, n, n).diagonal().array() += dks.ULSat(i, 0, 0, m + 1);
     }
-    scale_in_htil3_pjk_mE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
+    scale_in_hxxx3_pjk_mE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
     for(Index k = 1; k <= m; k++) {
         if(k % 100 == 0) {
             Rcpp::checkUserInterrupt();
@@ -1244,21 +1246,23 @@ htil3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
         go.block(0, 0, n, k * (p + 1)) = gn.block(0, 0, n, k * (p + 1));
         tG.noalias() = A2 * Go.block(0, 0, n, n);
         MatrixXx::Map(gn.col(0).data(), n, 1).noalias() =
-            (tG - Go.block(0, 0, n, n)) * mu + A2 * go.col(0);
+            (tG + c * Go.block(0, 0, n, n)) * mu + A2 * go.col(0);
         Gn.block(0, 0, n, n) = tG;
-        dks.ULSat(0, k, 0, m + 1) = (Gn.block(0, 0, n, n).trace() + gn.col(0).dot(mu)) / (2 * k);
+        dks.ULSat(0, k, 0, m + 1) =
+            (Gn.block(0, 0, n, n).trace() + gn.col(0).dot(mu)) / (2 * k);
         Gn.block(0, 0, n, n).diagonal().array() += dks.ULSat(0, k, 0, m + 1);
         for(Index i = 1; i <= p; i++) {
             tG.noalias() = A1 * Gn.block(0, (i - 1) * n, n, n) +
                            A2 * Go.block(0, i * n, n, n);
             MatrixXx::Map(gn.col(i).data(), n, 1).noalias() =
-                (tG - Go.block(0, i * n, n, n)) * mu +
+                (tG + c * Go.block(0, i * n, n, n)) * mu +
                 A1 * gn.col(i - 1) + A2 * go.col(i);
             Gn.block(0, i * n, n, n) = tG;
-            dks.ULSat(i, k, 0, m + 1) = (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * (k + i));
+            dks.ULSat(i, k, 0, m + 1) =
+                (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * (k + i));
             Gn.block(0, i * n, n, n).diagonal().array() += dks.ULSat(i, k, 0, m + 1);
         }
-        scale_in_htil3_pjk_mE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
+        scale_in_hxxx3_pjk_mE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
 #ifdef _OPENMP
 #pragma omp parallel private(tG, s2, s3)
 {
@@ -1270,56 +1274,55 @@ htil3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
             tG.noalias() = s2 * A2 * Go.block(0, j * n * (p + 1), n, n) +
                            s3 * A3 * Go.block(0, (j - 1) * n * (p + 1), n, n);
             MatrixXx::Map(gn.col(j * (p + 1)).data(), n, 1).noalias() =
-                (tG - s2 * Go.block(0, j * n * (p + 1), n, n) - s3 * Go.block(0, (j - 1) * n * (p + 1), n, n)) * mu +
+                (tG + c * (s2 * Go.block(0, j * n * (p + 1), n, n) + s3 * Go.block(0, (j - 1) * n * (p + 1), n, n))) * mu +
                 s2 * A2 * go.col(j * (p + 1)) + s3 * A3 * go.col((j - 1) * (p + 1));
             Gn.block(0, j * n * (p + 1), n, n) = tG;
-            dks.ULSat(0, k - j, j, m + 1) = (Gn.block(0, j * n * (p + 1), n, n).trace() + gn.col(j * (p + 1)).dot(mu)) / (2 * k);
+            dks.ULSat(0, k - j, j, m + 1) =
+                (Gn.block(0, j * n * (p + 1), n, n).trace() + gn.col(j * (p + 1)).dot(mu)) / (2 * k);
             Gn.block(0, j * n * (p + 1), n, n).diagonal().array() += dks.ULSat(0, k - j, j, m + 1);
             for(Index i = 1; i <= p; i++) {
                 tG.noalias() =      A1 * Gn.block(0, j * n * (p + 1) + (i - 1) * n, n, n) +
                                s2 * A2 * Go.block(0, j * n * (p + 1) + i * n, n, n) +
                                s3 * A3 * Go.block(0, (j - 1) * n * (p + 1) + i * n, n, n);
                 MatrixXx::Map(gn.col(j * (p + 1) + i).data(), n, 1).noalias() =
-                    (tG - s2 * Go.block(0, j * n * (p + 1) + i * n, n, n) - s3 * Go.block(0, (j - 1) * n * (p + 1) + i * n, n, n)) * mu +
+                    (tG + c * (s2 * Go.block(0, j * n * (p + 1) + i * n, n, n) +
+                               s3 * Go.block(0, (j - 1) * n * (p + 1) + i * n, n, n))) * mu +
                     A1 * gn.col(j * (p + 1) + i - 1) + s2 * A2 * go.col(j * (p + 1) + i) + s3 * A3 * go.col((j - 1) * (p + 1) + i);
                 Gn.block(0, j * n * (p + 1) + i * n, n, n) = tG;
-                dks.ULSat(i, k - j, j, m + 1) = (Gn.block(0, j * n * (p + 1) + i * n, n, n).trace() + gn.col(j * (p + 1) + i).dot(mu)) / (2 * (k + i));
+                dks.ULSat(i, k - j, j, m + 1) =
+                    (Gn.block(0, j * n * (p + 1) + i * n, n, n).trace() + gn.col(j * (p + 1) + i).dot(mu)) / (2 * (k + i));
                 Gn.block(0, j * n * (p + 1) + i * n, n, n).diagonal().array() += dks.ULSat(i, k - j, j, m + 1);
             }
-            scale_in_htil3_pjk_mE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
+            scale_in_hxxx3_pjk_mE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
         }
 #ifdef _OPENMP
 }
 #endif
         tG.noalias() = A3 * Go.block(0, (k - 1) * n * (p + 1), n, n);
         MatrixXx::Map(gn.col(k * (p + 1)).data(), n, 1).noalias() =
-            (tG - Go.block(0, (k - 1) * n * (p + 1), n, n)) * mu + A3 * go.col((k - 1) * (p + 1));
+            (tG + c * Go.block(0, (k - 1) * n * (p + 1), n, n)) * mu + A3 * go.col((k - 1) * (p + 1));
         Gn.block(0, k * n * (p + 1), n, n) = tG;
-        dks.ULSat(0, 0, k, m + 1) = (Gn.block(0, k * n * (p + 1), n, n).trace() + gn.col(k * (p + 1)).dot(mu)) / (2 * k);
+        dks.ULSat(0, 0, k, m + 1) =
+            (Gn.block(0, k * n * (p + 1), n, n).trace() + gn.col(k * (p + 1)).dot(mu)) / (2 * k);
         Gn.block(0, k * n * (p + 1), n, n).diagonal().array() += dks.ULSat(0, 0, k, m + 1);
         for(Index i = 1; i <= p; i++) {
             tG.noalias() = A1 * Gn.block(0, k * n * (p + 1) + (i - 1) * n, n, n) +
                            A3 * Go.block(0, (k - 1) * n * (p + 1) + i * n, n, n);
             MatrixXx::Map(gn.col(k * (p + 1) + i).data(), n, 1).noalias() =
-                (tG - Go.block(0, (k - 1) * n * (p + 1) + i * n, n, n)) * mu +
+                (tG + c * Go.block(0, (k - 1) * n * (p + 1) + i * n, n, n)) * mu +
                 A1 * gn.col(k * (p + 1) + i - 1) + A3 * go.col((k - 1) * (p + 1) + i);
             Gn.block(0, k * n * (p + 1) + i * n, n, n) = tG;
-            dks.ULSat(i, 0, k, m + 1) = (Gn.block(0, k * n * (p + 1) + i * n, n, n).trace() + gn.col(k * (p + 1) + i).dot(mu)) / (2 * (k + i));
+            dks.ULSat(i, 0, k, m + 1) =
+                (Gn.block(0, k * n * (p + 1) + i * n, n, n).trace() + gn.col(k * (p + 1) + i).dot(mu)) / (2 * (k + i));
             Gn.block(0, k * n * (p + 1) + i * n, n, n).diagonal().array() += dks.ULSat(i, 0, k, m + 1);
         }
-        scale_in_htil3_pjk_mE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
+        scale_in_hxxx3_pjk_mE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
     }
     return dks;
 }
-template ArrayXXd htil3_pjk_mEc(const MatrixBase<MatrixXd>& A1,
-                               const DiagMatXd& A2,
-                               const MatrixBase<MatrixXd>& A3,
-                               const VectorXd mu, const Index m, const Index p,
-                               ArrayXd& lscf, const double thr_margin,
-                               int nthreads);
 
 template <typename DerivedA, typename DerivedB, typename DerivedC>
-inline void scale_in_htil3_pjk_vE(Eigen::Index j, Eigen::Index k,
+inline void scale_in_hxxx3_pjk_vE(Eigen::Index j, Eigen::Index k,
                                   const Eigen::Index m, const Eigen::Index n,
                                   const Eigen::Index p,
                                   typename DerivedA::Scalar &thr,
@@ -1338,13 +1341,14 @@ inline void scale_in_htil3_pjk_vE(Eigen::Index j, Eigen::Index k,
 // // [[Rcpp::export]]
 template <typename Derived>
 Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, Eigen::Dynamic>
-htil3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
+hxxx3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
              const Eigen::ArrayBase<Derived>& A2,
              const Eigen::ArrayBase<Derived>& A3,
              const Eigen::ArrayBase<Derived>& mu,
              const Eigen::Index m, const Eigen::Index p,
              Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
-             const typename Derived::Scalar thr_margin, int nthreads) {
+             const typename Derived::Scalar thr_margin, int nthreads,
+             const typename Derived::Scalar c) {
 #ifdef _OPENMP
     if(nthreads <= 0) nthreads = std::max(omp_get_num_procs() / 2, 1);
     omp_set_num_threads(nthreads);
@@ -1365,9 +1369,10 @@ htil3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
     for(Index i = 1; i <= p; i++) {
         Gn.col(i) = A1 * (dks.ULSat(i - 1, 0, 0, m + 1) + Gn.col(i - 1));
         gn.col(i) = Gn.col(i) * mu + A1 * gn.col(i - 1);
-        dks.ULSat(i, 0, 0, m + 1) = (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * i);
+        dks.ULSat(i, 0, 0, m + 1) =
+            (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * i);
     }
-    scale_in_htil3_pjk_vE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
+    scale_in_hxxx3_pjk_vE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
     for(Index k = 1; k <= m; k++) {
         if(k % 500 == 0) {
             Rcpp::checkUserInterrupt();
@@ -1375,19 +1380,23 @@ htil3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
         Go.block(0, 0, n, k * (p + 1)) = Gn.block(0, 0, n, k * (p + 1));
         go.block(0, 0, n, k * (p + 1)) = gn.block(0, 0, n, k * (p + 1));
         tG = A2 * (dks.ULSat(0, k - 1, 0, m + 1) + Go.col(0));
-        gn.col(0) = (tG - Go.col(0) - (dks.ULSat(0, k - 1, 0, m + 1))) * mu + A2 * go.col(0);
+        gn.col(0) =
+            (tG + c * Go.col(0) + c * dks.ULSat(0, k - 1, 0, m + 1)) * mu +
+            A2 * go.col(0);
         Gn.col(0) = tG;
-        dks.ULSat(0, k, 0, m + 1) = (Gn.col(0).sum() + (mu * gn.col(0)).sum()) / (2 * k);
+        dks.ULSat(0, k, 0, m + 1) =
+            (Gn.col(0).sum() + (mu * gn.col(0)).sum()) / (2 * k);
         for(Index i = 1; i <= p; i++) {
             tG = A1 * (dks.ULSat(i - 1, k, 0, m + 1) + Gn.col(i - 1)) +
                  A2 * (dks.ULSat(i, k - 1, 0, m + 1) + Go.col(i));
-            gn.col(i) = (tG - Go.col(i)
-                         - (dks.ULSat(i, k - 1, 0, m + 1))) * mu +
+            gn.col(i) = (tG + c * Go.col(i)
+                         + c * dks.ULSat(i, k - 1, 0, m + 1)) * mu +
                         A1 * gn.col(i - 1) + A2 * go.col(i);
             Gn.col(i) = tG;
-            dks.ULSat(i, k, 0, m + 1) = (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * (k + i));
+            dks.ULSat(i, k, 0, m + 1) =
+                (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * (k + i));
         }
-        scale_in_htil3_pjk_vE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
+        scale_in_hxxx3_pjk_vE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
 #ifdef _OPENMP
 #pragma omp parallel private(tG, s2, s3)
 {
@@ -1399,44 +1408,84 @@ htil3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
             tG = s2 * A2 * (dks.ULSat(0, k - j - 1, j, m + 1) + Go.col(j * (p + 1))) +
                  s3 * A3 * (dks.ULSat(0, k - j, j - 1, m + 1) + Go.col((j - 1) * (p + 1)));
             gn.col(j * (p + 1)) =
-                (tG - s2 * Go.col(j * (p + 1)) - s3 * Go.col((j - 1) * (p + 1)) - ((s2 * dks.ULSat(0, k - j - 1, j, m + 1) + s3 * dks.ULSat(0, k - j, j - 1, m + 1)))) * mu +
+                (tG + c * (s2 * Go.col(j * (p + 1)) + s3 * Go.col((j - 1) * (p + 1)))
+                    + c * (s2 * dks.ULSat(0, k - j - 1, j, m + 1) + s3 * dks.ULSat(0, k - j, j - 1, m + 1))) * mu +
                 s2 * A2 * go.col(j * (p + 1)) + s3 * A3 * go.col((j - 1) * (p + 1));
             Gn.col(j * (p + 1)) = tG;
-            dks.ULSat(0, k - j, j, m + 1) = (Gn.col(j * (p + 1)).sum() + (mu * gn.col(j * (p + 1))).sum()) / (2 * k);
+            dks.ULSat(0, k - j, j, m + 1) =
+                (Gn.col(j * (p + 1)).sum() + (mu * gn.col(j * (p + 1))).sum()) / (2 * k);
             for(Index i = 1; i <= p; i++) {
                 tG =      A1 * (dks.ULSat(i - 1, k - j, j, m + 1) + Gn.col(j * (p + 1) + i - 1)) +
                      s2 * A2 * (dks.ULSat(i, k - j - 1, j, m + 1) + Go.col(j * (p + 1) + i)) +
                      s3 * A3 * (dks.ULSat(i, k - j, j - 1, m + 1) + Go.col((j - 1) * (p + 1) + i));
-                gn.col(j * (p + 1) + i) = (tG - s2 * Go.col(j * (p + 1) + i) - s3 * Go.col((j - 1) * (p + 1) + i)
-                             - ((s2 * dks.ULSat(i, k - j - 1, j, m + 1) + s3 * dks.ULSat(i, k - j, j - 1, m + 1)))) * mu +
-                            A1 * gn.col(j * (p + 1) + i - 1) + s2 * A2 * go.col(j * (p + 1) + i) + s3 * A3 * go.col((j - 1) * (p + 1) + i);
+                gn.col(j * (p + 1) + i) =
+                    (tG + c * (s2 * Go.col(j * (p + 1) + i) + s3 * Go.col((j - 1) * (p + 1) + i))
+                        + c * (s2 * dks.ULSat(i, k - j - 1, j, m + 1) + s3 * dks.ULSat(i, k - j, j - 1, m + 1))) * mu +
+                    A1 * gn.col(j * (p + 1) + i - 1) + s2 * A2 * go.col(j * (p + 1) + i) + s3 * A3 * go.col((j - 1) * (p + 1) + i);
                 Gn.col(j * (p + 1) + i) = tG;
-                dks.ULSat(i, k - j, j, m + 1) = (Gn.col(j * (p + 1) + i).sum() + (mu * gn.col(j * (p + 1) + i)).sum()) / (2 * (k + i));
+                dks.ULSat(i, k - j, j, m + 1) =
+                    (Gn.col(j * (p + 1) + i).sum() + (mu * gn.col(j * (p + 1) + i)).sum()) / (2 * (k + i));
             }
-            scale_in_htil3_pjk_vE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
+            scale_in_hxxx3_pjk_vE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
         }
 #ifdef _OPENMP
 }
 #endif
         tG = A3 * (dks.ULSat(0, 0, k - 1, m + 1) + Go.col((k - 1) * (p + 1)));
         gn.col(k * (p + 1)) =
-            (tG - Go.col((k - 1) * (p + 1)) - (dks.ULSat(0, 0, k - 1, m + 1))) * mu +
+            (tG + c * Go.col((k - 1) * (p + 1))
+                + c * dks.ULSat(0, 0, k - 1, m + 1)) * mu +
             A3 * go.col((k - 1) * (p + 1));
         Gn.col(k * (p + 1)) = tG;
-        dks.ULSat(0, 0, k, m + 1) = (Gn.col(k * (p + 1)).sum() + (mu * gn.col(k * (p + 1))).sum()) / (2 * k);
+        dks.ULSat(0, 0, k, m + 1) =
+            (Gn.col(k * (p + 1)).sum() + (mu * gn.col(k * (p + 1))).sum()) / (2 * k);
         for(Index i = 1; i <= p; i++) {
             tG = A1 * (dks.ULSat(i - 1, 0, k, m + 1) + Gn.col(k * (p + 1) + i - 1)) +
                  A3 * (dks.ULSat(i, 0, k - 1, m + 1) + Go.col((k - 1) * (p + 1) + i));
-            gn.col(k * (p + 1) + i) = (tG - Go.col((k - 1) * (p + 1) + i)
-                         - (dks.ULSat(i, 0, k - 1, m + 1))) * mu +
-                        A1 * gn.col(k * (p + 1) + i - 1) + A3 * go.col((k - 1) * (p + 1) + i);
+            gn.col(k * (p + 1) + i) =
+                (tG + c * Go.col((k - 1) * (p + 1) + i)
+                    + c * dks.ULSat(i, 0, k - 1, m + 1)) * mu +
+                A1 * gn.col(k * (p + 1) + i - 1) + A3 * go.col((k - 1) * (p + 1) + i);
             Gn.col(k * (p + 1) + i) = tG;
             dks.ULSat(i, 0, k, m + 1) = (Gn.col(k * (p + 1) + i).sum() + (mu * gn.col(k * (p + 1) + i)).sum()) / (2 * (k + i));
         }
-        scale_in_htil3_pjk_vE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
+        scale_in_hxxx3_pjk_vE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
     }
     return dks;
 }
+
+// // [[Rcpp::export]]
+template <typename Derived>
+Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, Eigen::Dynamic>
+htil3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
+             const Eigen::DiagonalMatrix<typename Derived::Scalar, Eigen::Dynamic>& A2,
+             const Eigen::MatrixBase<Derived>& A3,
+             const Eigen::Matrix<typename Derived::Scalar, Eigen::Dynamic, 1> mu,
+             const Eigen::Index m, const Eigen::Index p,
+             Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
+             const typename Derived::Scalar thr_margin, int nthreads) {
+    return hxxx3_pjk_mEc(A1, A2, A3, mu, m, p, lscf, thr_margin, nthreads, -1);
+}
+template ArrayXXd htil3_pjk_mEc(const MatrixBase<MatrixXd>& A1,
+                               const DiagMatXd& A2,
+                               const MatrixBase<MatrixXd>& A3,
+                               const VectorXd mu, const Index m, const Index p,
+                               ArrayXd& lscf, const double thr_margin,
+                               int nthreads);
+
+
+// // [[Rcpp::export]]
+template <typename Derived>
+Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, Eigen::Dynamic>
+htil3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
+             const Eigen::ArrayBase<Derived>& A2,
+             const Eigen::ArrayBase<Derived>& A3,
+             const Eigen::ArrayBase<Derived>& mu,
+             const Eigen::Index m, const Eigen::Index p,
+             Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
+             const typename Derived::Scalar thr_margin, int nthreads) {
+    return hxxx3_pjk_vEc(A1, A2, A3, mu, m, p, lscf, thr_margin, nthreads, -1);
+}
 template ArrayXXd htil3_pjk_vEc(const ArrayBase<ArrayXd>& A1,
                                const ArrayBase<ArrayXd>& A2,
                                const ArrayBase<ArrayXd>& A3,
@@ -1454,107 +1503,7 @@ hhat3_pjk_mEc(const Eigen::MatrixBase<Derived>& A1,
              const Eigen::Index m, const Eigen::Index p,
              Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
              const typename Derived::Scalar thr_margin, int nthreads) {
-#ifdef _OPENMP
-    if(nthreads <= 0) nthreads = std::max(omp_get_num_procs() / 2, 1);
-    omp_set_num_threads(nthreads);
-#endif
-    typedef typename Derived::Scalar Scalar;
-    typedef Matrix<Scalar, Dynamic, Dynamic> MatrixXx;
-    typedef Array<Scalar, Dynamic, Dynamic> ArrayXXx;
-    const Index n = A1.rows();
-    ArrayXXx dks = ArrayXXx::Zero(p + 1, (m + 1) * (m + 2) / 2);
-    dks.ULSat(0, 0, 0, m + 1) = 1;
-    Scalar thr = std::numeric_limits<Scalar>::max() / thr_margin / Scalar(n);
-    MatrixXx tG(n, n);
-    MatrixXx Go = MatrixXx::Zero(n, n * (p + 1) * m);
-    MatrixXx Gn = MatrixXx::Zero(n, n * (p + 1) * (m + 1));
-    MatrixXx go = MatrixXx::Zero(n, (p + 1) * m);
-    MatrixXx gn = MatrixXx::Zero(n, (p + 1) * (m + 1));
-    Gn.block(0, 0, n, n).diagonal().array() += dks.ULSat(0, 0, 0, m + 1);
-    Scalar s2, s3;
-    for(Index i = 1; i <= p; i++) {
-        MatrixXx::Map(Gn.block(0, i * n, n, n).data(), n, n).noalias() =
-            A1 * Gn.block(0, (i - 1) * n, n, n);
-        MatrixXx::Map(gn.col(i).data(), n, 1).noalias() =
-            Gn.block(0, i * n, n, n) * mu + A1 * gn.col(i - 1);
-        dks.ULSat(i, 0, 0, m + 1) = (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * i);
-        Gn.block(0, i * n, n, n).diagonal().array() += dks.ULSat(i, 0, 0, m + 1);
-    }
-    scale_in_htil3_pjk_mE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
-    for(Index k = 1; k <= m; k++) {
-        if(k % 100 == 0) {
-            Rcpp::checkUserInterrupt();
-        }
-        Go.block(0, 0, n, k * n * (p + 1)) = Gn.block(0, 0, n, k * n * (p + 1));
-        go.block(0, 0, n, k * (p + 1)) = gn.block(0, 0, n, k * (p + 1));
-        tG.noalias() = A2 * Go.block(0, 0, n, n);
-        MatrixXx::Map(gn.col(0).data(), n, 1).noalias() =
-            (tG + Go.block(0, 0, n, n)) * mu + A2 * go.col(0);
-        Gn.block(0, 0, n, n) = tG;
-        dks.ULSat(0, k, 0, m + 1) = (Gn.block(0, 0, n, n).trace() + gn.col(0).dot(mu)) / (2 * k);
-        Gn.block(0, 0, n, n).diagonal().array() += dks.ULSat(0, k, 0, m + 1);
-        for(Index i = 1; i <= p; i++) {
-            tG.noalias() = A1 * Gn.block(0, (i - 1) * n, n, n) +
-                           A2 * Go.block(0, i * n, n, n);
-            MatrixXx::Map(gn.col(i).data(), n, 1).noalias() =
-                (tG + Go.block(0, i * n, n, n)) * mu +
-                A1 * gn.col(i - 1) + A2 * go.col(i);
-            Gn.block(0, i * n, n, n) = tG;
-            dks.ULSat(i, k, 0, m + 1) = (Gn.block(0, i * n, n, n).trace() + gn.col(i).dot(mu)) / (2 * (k + i));
-            Gn.block(0, i * n, n, n).diagonal().array() += dks.ULSat(i, k, 0, m + 1);
-        }
-        scale_in_htil3_pjk_mE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
-#ifdef _OPENMP
-#pragma omp parallel private(tG, s2, s3)
-{
-#pragma omp for
-#endif
-        for(Index j = 1; j < k; j++) {
-            s2 = exp(min<Scalar>(0, lscf.ULTat(k - j, j - 1, m + 1) - lscf.ULTat(k - j - 1, j, m + 1)));
-            s3 = exp(min<Scalar>(0, lscf.ULTat(k - j - 1, j, m + 1) - lscf.ULTat(k - j, j - 1, m + 1)));
-            tG.noalias() = s2 * A2 * Go.block(0, j * n * (p + 1), n, n) +
-                           s3 * A3 * Go.block(0, (j - 1) * n * (p + 1), n, n);
-            MatrixXx::Map(gn.col(j * (p + 1)).data(), n, 1).noalias() =
-                (tG + s2 * Go.block(0, j * n * (p + 1), n, n) + s3 * Go.block(0, (j - 1) * n * (p + 1), n, n)) * mu +
-                s2 * A2 * go.col(j * (p + 1)) + s3 * A3 * go.col((j - 1) * (p + 1));
-            Gn.block(0, j * n * (p + 1), n, n) = tG;
-            dks.ULSat(0, k - j, j, m + 1) = (Gn.block(0, j * n * (p + 1), n, n).trace() + gn.col(j * (p + 1)).dot(mu)) / (2 * k);
-            Gn.block(0, j * n * (p + 1), n, n).diagonal().array() += dks.ULSat(0, k - j, j, m + 1);
-            for(Index i = 1; i <= p; i++) {
-                tG.noalias() =      A1 * Gn.block(0, j * n * (p + 1) + (i - 1) * n, n, n) +
-                               s2 * A2 * Go.block(0, j * n * (p + 1) + i * n, n, n) +
-                               s3 * A3 * Go.block(0, (j - 1) * n * (p + 1) + i * n, n, n);
-                MatrixXx::Map(gn.col(j * (p + 1) + i).data(), n, 1).noalias() =
-                    (tG + s2 * Go.block(0, j * n * (p + 1) + i * n, n, n) + s3 * Go.block(0, (j - 1) * n * (p + 1) + i * n, n, n)) * mu +
-                    A1 * gn.col(j * (p + 1) + i - 1) + s2 * A2 * go.col(j * (p + 1) + i) + s3 * A3 * go.col((j - 1) * (p + 1) + i);
-                Gn.block(0, j * n * (p + 1) + i * n, n, n) = tG;
-                dks.ULSat(i, k - j, j, m + 1) = (Gn.block(0, j * n * (p + 1) + i * n, n, n).trace() + gn.col(j * (p + 1) + i).dot(mu)) / (2 * (k + i));
-                Gn.block(0, j * n * (p + 1) + i * n, n, n).diagonal().array() += dks.ULSat(i, k - j, j, m + 1);
-            }
-            scale_in_htil3_pjk_mE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
-        }
-#ifdef _OPENMP
-}
-#endif
-        tG.noalias() = A3 * Go.block(0, (k - 1) * n * (p + 1), n, n);
-        MatrixXx::Map(gn.col(k * (p + 1)).data(), n, 1).noalias() =
-            (tG + Go.block(0, (k - 1) * n * (p + 1), n, n)) * mu + A3 * go.col((k - 1) * (p + 1));
-        Gn.block(0, k * n * (p + 1), n, n) = tG;
-        dks.ULSat(0, 0, k, m + 1) = (Gn.block(0, k * n * (p + 1), n, n).trace() + gn.col(k * (p + 1)).dot(mu)) / (2 * k);
-        Gn.block(0, k * n * (p + 1), n, n).diagonal().array() += dks.ULSat(0, 0, k, m + 1);
-        for(Index i = 1; i <= p; i++) {
-            tG.noalias() = A1 * Gn.block(0, k * n * (p + 1) + (i - 1) * n, n, n) +
-                           A3 * Go.block(0, (k - 1) * n * (p + 1) + i * n, n, n);
-            MatrixXx::Map(gn.col(k * (p + 1) + i).data(), n, 1).noalias() =
-                (tG + Go.block(0, (k - 1) * n * (p + 1) + i * n, n, n)) * mu +
-                A1 * gn.col(k * (p + 1) + i - 1) + A3 * go.col((k - 1) * (p + 1) + i);
-            Gn.block(0, k * n * (p + 1) + i * n, n, n) = tG;
-            dks.ULSat(i, 0, k, m + 1) = (Gn.block(0, k * n * (p + 1) + i * n, n, n).trace() + gn.col(k * (p + 1) + i).dot(mu)) / (2 * (k + i));
-            Gn.block(0, k * n * (p + 1) + i * n, n, n).diagonal().array() += dks.ULSat(i, 0, k, m + 1);
-        }
-        scale_in_htil3_pjk_mE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
-    }
-    return dks;
+    return hxxx3_pjk_mEc(A1, A2, A3, mu, m, p, lscf, thr_margin, nthreads, 1);
 }
 template ArrayXXd hhat3_pjk_mEc(const MatrixBase<MatrixXd>& A1,
                                const DiagMatXd& A2,
@@ -1573,99 +1522,7 @@ hhat3_pjk_vEc(const Eigen::ArrayBase<Derived>& A1,
              const Eigen::Index m, const Eigen::Index p,
              Eigen::Array<typename Derived::Scalar, Eigen::Dynamic, 1>& lscf,
              const typename Derived::Scalar thr_margin, int nthreads) {
-#ifdef _OPENMP
-    if(nthreads <= 0) nthreads = std::max(omp_get_num_procs() / 2, 1);
-    omp_set_num_threads(nthreads);
-#endif
-    typedef typename Derived::Scalar Scalar;
-    typedef Array<Scalar, Dynamic, 1> ArrayXx;
-    typedef Array<Scalar, Dynamic, Dynamic> ArrayXXx;
-    const Index n = A1.rows();
-    ArrayXXx dks = ArrayXXx::Zero(p + 1, (m + 1) * (m + 2) / 2);
-    dks.ULSat(0, 0, 0, m + 1) = 1;
-    Scalar thr = std::numeric_limits<Scalar>::max() / thr_margin / Scalar(n);
-    ArrayXx tG(n);
-    ArrayXXx Go = ArrayXXx::Zero(n, (p + 1) * m);
-    ArrayXXx Gn = ArrayXXx::Zero(n, (p + 1) * (m + 1));
-    ArrayXXx go = ArrayXXx::Zero(n, (p + 1) * m);
-    ArrayXXx gn = ArrayXXx::Zero(n, (p + 1) * (m + 1));
-    Scalar s2, s3;
-    for(Index i = 1; i <= p; i++) {
-        Gn.col(i) = A1 * (dks.ULSat(i - 1, 0, 0, m + 1) + Gn.col(i - 1));
-        gn.col(i) = Gn.col(i) * mu + A1 * gn.col(i - 1);
-        dks.ULSat(i, 0, 0, m + 1) = (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * i);
-    }
-    scale_in_htil3_pjk_vE(0, 0, m, n, p, thr, dks, lscf, Gn, gn);
-    for(Index k = 1; k <= m; k++) {
-        if(k % 500 == 0) {
-            Rcpp::checkUserInterrupt();
-        }
-        Go.block(0, 0, n, k * (p + 1)) = Gn.block(0, 0, n, k * (p + 1));
-        go.block(0, 0, n, k * (p + 1)) = gn.block(0, 0, n, k * (p + 1));
-        tG = A2 * (dks.ULSat(0, k - 1, 0, m + 1) + Go.col(0));
-        gn.col(0) = (tG + Go.col(0) + (dks.ULSat(0, k - 1, 0, m + 1))) * mu + A2 * go.col(0);
-        Gn.col(0) = tG;
-        dks.ULSat(0, k, 0, m + 1) = (Gn.col(0).sum() + (mu * gn.col(0)).sum()) / (2 * k);
-        for(Index i = 1; i <= p; i++) {
-            tG = A1 * (dks.ULSat(i - 1, k, 0, m + 1) + Gn.col(i - 1)) +
-                 A2 * (dks.ULSat(i, k - 1, 0, m + 1) + Go.col(i));
-            gn.col(i) = (tG + Go.col(i)
-                         + (dks.ULSat(i, k - 1, 0, m + 1))) * mu +
-                        A1 * gn.col(i - 1) + A2 * go.col(i);
-            Gn.col(i) = tG;
-            dks.ULSat(i, k, 0, m + 1) = (Gn.col(i).sum() + (mu * gn.col(i)).sum()) / (2 * (k + i));
-        }
-        scale_in_htil3_pjk_vE(0, k, m, n, p, thr, dks, lscf, Gn, gn);
-#ifdef _OPENMP
-#pragma omp parallel private(tG, s2, s3)
-{
-#pragma omp for
-#endif
-        for(Index j = 1; j < k; j++) {
-            s2 = exp(min<Scalar>(0, lscf.ULTat(k - j, j - 1, m + 1) - lscf.ULTat(k - j - 1, j, m + 1)));
-            s3 = exp(min<Scalar>(0, lscf.ULTat(k - j - 1, j, m + 1) - lscf.ULTat(k - j, j - 1, m + 1)));
-            tG = s2 * A2 * (dks.ULSat(0, k - j - 1, j, m + 1) + Go.col(j * (p + 1))) + s3 * A3 * (dks.ULSat(0, k - j, j - 1, m + 1) + Go.col((j - 1) * (p + 1)));
-            gn.col(j * (p + 1)) =
-                (tG + s2 * Go.col(j * (p + 1)) + s3 * Go.col((j - 1) * (p + 1)) +
-                 ((s2 * dks.ULSat(0, k - j - 1, j, m + 1) + s3 * dks.ULSat(0, k - j, j - 1, m + 1)))) * mu +
-                s2 * A2 * go.col(j * (p + 1)) + s3 * A3 * go.col((j - 1) * (p + 1));
-            Gn.col(j * (p + 1)) = tG;
-            dks.ULSat(0, k - j, j, m + 1) = (Gn.col(j * (p + 1)).sum() + (mu * gn.col(j * (p + 1))).sum()) / (2 * k);
-            for(Index i = 1; i <= p; i++) {
-                tG =      A1 * (dks.ULSat(i - 1, k - j, j, m + 1) + Gn.col(j * (p + 1) + i - 1)) +
-                     s2 * A2 * (dks.ULSat(i, k - j - 1, j, m + 1) + Go.col(j * (p + 1) + i)) +
-                     s3 * A3 * (dks.ULSat(i, k - j, j - 1, m + 1) + Go.col((j - 1) * (p + 1) + i));
-                gn.col(j * (p + 1) + i) =
-                    (tG + s2 * Go.col(j * (p + 1) + i) + s3 * Go.col((j - 1) * (p + 1) + i)
-                     + ((s2 * dks.ULSat(i, k - j - 1, j, m + 1) + s3 * dks.ULSat(i, k - j, j - 1, m + 1)))) * mu +
-                    A1 * gn.col(j * (p + 1) + i - 1) + s2 * A2 * go.col(j * (p + 1) + i) + s3 * A3 * go.col((j - 1) * (p + 1) + i);
-                Gn.col(j * (p + 1) + i) = tG;
-                dks.ULSat(i, k - j, j, m + 1) = (Gn.col(j * (p + 1) + i).sum() + (mu * gn.col(j * (p + 1) + i)).sum()) / (2 * (k + i));
-            }
-            scale_in_htil3_pjk_vE(j, k, m, n, p, thr, dks, lscf, Gn, gn);
-        }
-#ifdef _OPENMP
-}
-#endif
-        tG = A3 * (dks.ULSat(0, 0, k - 1, m + 1) + Go.col((k - 1) * (p + 1)));
-        gn.col(k * (p + 1)) =
-            (tG + Go.col((k - 1) * (p + 1)) + (dks.ULSat(0, 0, k - 1, m + 1))) * mu +
-            A3 * go.col((k - 1) * (p + 1));
-        Gn.col(k * (p + 1)) = tG;
-        dks.ULSat(0, 0, k, m + 1) = (Gn.col(k * (p + 1)).sum() + (mu * gn.col(k * (p + 1))).sum()) / (2 * k);
-        for(Index i = 1; i <= p; i++) {
-            tG = A1 * (dks.ULSat(i - 1, 0, k, m + 1) + Gn.col(k * (p + 1) + i - 1)) +
-                 A3 * (dks.ULSat(i, 0, k - 1, m + 1) + Go.col((k - 1) * (p + 1) + i));
-            gn.col(k * (p + 1) + i) =
-                (tG + Go.col((k - 1) * (p + 1) + i)
-                 + (dks.ULSat(i, 0, k - 1, m + 1))) * mu +
-                A1 * gn.col(k * (p + 1) + i - 1) + A3 * go.col((k - 1) * (p + 1) + i);
-            Gn.col(k * (p + 1) + i) = tG;
-            dks.ULSat(i, 0, k, m + 1) = (Gn.col(k * (p + 1) + i).sum() + (mu * gn.col(k * (p + 1) + i)).sum()) / (2 * (k + i));
-        }
-        scale_in_htil3_pjk_vE(k, k, m, n, p, thr, dks, lscf, Gn, gn);
-    }
-    return dks;
+    return hxxx3_pjk_vEc(A1, A2, A3, mu, m, p, lscf, thr_margin, nthreads, 1);
 }
 template ArrayXXd hhat3_pjk_vEc(const ArrayBase<ArrayXd>& A1,
                                const ArrayBase<ArrayXd>& A2,
